import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Basisklasse fr alle Tiere wie Kara, Ameisen, etc.
 * Tiere knnen sich in der Welt bewegen, das heit vorwrts bewegen oder sich drehen.
 * 
 * @author Daniel Garmann
 * @version 09.08.2009
 */
public class Tier extends Actor
{   
   
    /**
     * Konstruktor eines Tiers. Nichts zu tun, da jedes Tier individuelle Konstruktoren hat.
     */
    public Tier() {
        // nichts zu tun.
    }
    
    /**
     * Act - Hier wird noch nichts gemacht, da das konkrete Verhalten vom Tier abhngig ist.
     */
    public void act() 
    {
        // Nichts zu tun.
    }    
    
    /**
     * Drehung um einen Winkel gegen den Uhrzeigersinn
     * @param angle - Drehwinkel in Degree
     */
    public void turn(int angle) {
        this.setRotation(this.getRotation() - angle);
    }
        
    
    /**
     * Vorwrtsbewegung in die aktuelle Richtung
     */
    public void move(double speed)
    {
        double angle = Math.toRadians( getRotation() );
        int x = (int) Math.round(getX() + Math.cos(angle) * speed);
        x = (x + getWorld().getWidth()) % getWorld().getWidth();
        int y = (int) Math.round(getY() + Math.sin(angle) * speed);
        y = (y + getWorld().getHeight()) % getWorld().getHeight();        
        setLocation(x, y);
    }
    

    /**
     * Prfen, ob ein Objekt in der Nhe des Tiers ist.
     * @param clss Klasse, welches Objekt gesehen werden soll.
     * @return true wenn das Objekt ein anderes Objekt der Klasse 'clss' sehen kann, andernfalls false 
     */
    public boolean canSee(Class clss)
    {
        Actor actor = getOneObjectAtOffset(0, 0, clss);
        return actor != null;        
    }
    
    /**
     * Versucht ein Objekt der Klasse clss zu fressen, welches in direkter Nachbarschaft liegt.
     * Ist kein Objekt der Klasse vorhanden, so passiert nichts.
     * @param clss Klasse, welches Objekt gefressen werden soll.
     */
    public void eat(Class clss)
    {
        Actor actor = getOneObjectAtOffset(0, 0, clss);
        if(actor != null) {
            getWorld().removeObject(actor);
        }
    }
    
    /**
     * Versucht ein Objekt der Klasse clss abzulegen.
     * @param clss Klasse, von der ein Objekt erzeugt und abgelegt werden soll.
     */
    public void put(Class clss)
    {
        try {
            Actor actor = (Actor)clss.newInstance();
            getWorld().addObject(actor, this.getX(), this.getY());
            
        } catch (Exception e) {
            System.out.println(e.toString());
        }
    }
    
    
}
